package com.github.jfcloud.gene.constants;

import cn.hutool.core.lang.Pair;

import java.util.Arrays;
import java.util.List;

/**
 * 基因定制流程状态
 */
public enum GeneStatusEnum {
    DRAFT("0", "待提交"),
    REJECTED("-1", "驳回"),
    GENE_EDITING("20", "基因编辑部门负责人审核"),
    DEPART_LEADER("25", "部门负责人审核"),
    PROJECT_LEADER("40", "项目负责人审核"),
    PROJECT_MANAGEMENT("60", "项目管理部审核"),
    COMPLETED("100", "已完成");

    private final String status;
    private final String description;

    GeneStatusEnum(String status, String description) {
        this.status = status;
        this.description = description;
    }

    /**
     * 提交状态
     */
    public static final List<String> SUBMIT_STATUS = Arrays.asList(DRAFT.getStatus(), REJECTED.getStatus());

    public String getStatus() {
        return status;
    }

    public String getDescription() {
        return description;
    }

    public Pair<String, String> toPair() {
        return new Pair<>(status, description);
    }

    public static GeneStatusEnum getByStatus(String status) {
        for (GeneStatusEnum e : values()) {
            if (e.getStatus().equals(status)) {
                return e;
            }
        }
        return null;
    }

    /**
     * 基因定制流程状态列表
     */
    public static List<Pair<String, String>> getGeneStatusList() {
        return Arrays.asList(DRAFT.toPair(),
                REJECTED.toPair(),
                GENE_EDITING.toPair(),
                PROJECT_LEADER.toPair(),
                PROJECT_MANAGEMENT.toPair(),
                COMPLETED.toPair()
        );
    }

    /**
     * 样本送检流程状态列表
     */
    public static List<Pair<String, String>> getSampleStatusList() {
        return Arrays.asList(DRAFT.toPair(),
                REJECTED.toPair(),
                DEPART_LEADER.toPair(),
                PROJECT_MANAGEMENT.toPair(),
                COMPLETED.toPair()
        );
    }
}
