package com.github.jfcloud.gene.file.controller;

import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.github.jfcloud.common.core.util.R;
import com.github.jfcloud.common.file.core.FileProperties;
import com.github.jfcloud.common.file.core.FileTemplate;
import com.github.jfcloud.gene.file.vo.FileVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;

@Slf4j
@Tag(name = "文件")
@RestController
@RequestMapping("/file")
@RequiredArgsConstructor
public class FileController {

    private final FileProperties ossProperties;
    private final FileTemplate fileTemplate;

    @Operation(summary = "上传文件")
    @PostMapping(value = "/upload")
    public R<FileVo> upload(@RequestPart("file") MultipartFile file) {
        try (InputStream in = file.getInputStream()) {
            String fileName = IdUtil.simpleUUID() + StrUtil.DOT + FileNameUtil.getSuffix(file.getOriginalFilename());
            fileTemplate.putObject(ossProperties.getBucketName(), fileName, in);

            FileVo vo = new FileVo();
            vo.setName(file.getOriginalFilename());
            vo.setUrl(String.format("/admin/sys-file/%s/%s", ossProperties.getBucketName(), fileName));
            return R.ok(vo);

        } catch (Exception e) {
            log.error("上传失败", e);
        }
        return R.failed();
    }
}
