package com.github.jfcloud.gene.file.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.io.unit.DataSizeUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.alibaba.fastjson.JSONObject;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.S3ObjectSummary;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.common.file.core.FileProperties;
import com.github.jfcloud.common.file.core.FileTemplate;
import com.github.jfcloud.common.holder.RequestHolder;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.file.entity.FileInfo;
import com.github.jfcloud.gene.file.mapper.FileInfoMapper;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.file.vo.FileDetail;
import com.github.jfcloud.gene.file.vo.FileVo;
import com.github.jfcloud.gene.flow.entity.FlowFileVersion;
import com.github.jfcloud.gene.util.WordUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.docx4j.Docx4J;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class FileInfoServiceImpl extends ServiceImpl<FileInfoMapper, FileInfo> implements FileInfoService {

    private final FileTemplate fileTemplate;

    private final FileProperties ossProperties;

    @Override
    public List<FileVo> listByRelateId(Long relateId) {
        List<FileInfo> list = list(new LambdaQueryWrapper<>(FileInfo.class)
                .eq(FileInfo::getRelateId, relateId)
                .eq(FileInfo::getDeleted, WhetherEnum.NO.getCode())
                .orderByAsc(FileInfo::getId));
        return BeanUtil.copyToList(list, FileVo.class);
    }

    public FileVo uploadFileWithFileName(InputStream fileInputStream, String fileName) {
        Assert.notNull(fileInputStream, "FileInputStream can not be null");

        FileVo vo = new FileVo();
        vo.setName(fileName);

        try {
            String bucketName = ossProperties.getBucketName();
            fileTemplate.removeObject(bucketName, fileName);
            fileTemplate.putObject(bucketName, fileName, fileInputStream);

            vo.setUrl(String.format("/admin/sys-file/%s/%s", bucketName, fileName));
        } catch (Exception e) {
            log.error("上传失败", e);
            Assert.isTrue(false, "附件生成上传失败");
        }
        return vo;
    }

    @Override
    public void uploadFileWithFileName(String modelName, String fileName, Long flowId, String flowStatus, JSONObject dataMap) {
        log.info("流程id={} status={} 模板modelName={} 生成文档：{}", flowId, flowStatus, modelName, fileName);

        try (InputStream inputStream = WordUtil.exportWord(modelName, dataMap);
             ByteArrayOutputStream out = new ByteArrayOutputStream()) {
            //转化为docx文件
            Docx4J.save(Docx4J.load(inputStream), out);

            //上传至文件服务器
            byte[] byteArray = out.toByteArray();
            try (ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(byteArray)) {
                FileVo fileVo = uploadFileWithFileName(byteArrayInputStream, fileName);

                HttpServletRequest request = RequestHolder.getRequest();
                String uri = request.getMethod() + " " + request.getRequestURI();

                new FlowFileVersion()
                        .setFlowId(flowId)
                        .setFlowStatus(flowStatus)
                        .setDataMapWithoutImg(dataMap.toJSONString())
                        .setFileMd5(DigestUtil.md5Hex(byteArray))
                        .setFilename(fileVo.getName())
                        .setFilepath(fileVo.getUrl())
                        .setUri(uri)
                        .setTemplateName(modelName)
                        .insert();
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("附件生成失败");
        }
    }

    @Override
    public FileDetail getFileSize(String url) {
        String uri = url.replace("/admin/sys-file/", "");
        String bucketName = uri.substring(0, uri.indexOf("/"));
        String filename = uri.substring(uri.indexOf("/") + 1);

        FileDetail fileDetail = new FileDetail()
                .setBucketName(bucketName)
                .setFilename(filename);

        //查询文件大小
        AmazonS3 amazonS3 = (AmazonS3) ReflectUtil.getFieldValue(fileTemplate, "amazonS3");
        List<S3ObjectSummary> apks = amazonS3.listObjects(bucketName, filename).getObjectSummaries();
        if (!apks.isEmpty()) {
            long contentLength = apks.get(0).getSize();

            return fileDetail
                    .setLength(contentLength)
                    .setFileSize(DataSizeUtil.format(contentLength));
        }
        return fileDetail;
    }
}
