package com.github.jfcloud.gene.flow.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.admin.api.sys.dto.message.MessageUserDTO;
import com.github.jfcloud.common.core.util.R;
import com.github.jfcloud.common.data.datascope.DataScope;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.constant.StrConstant;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.common.util.UserUtil;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.constants.GeneTargetEnum;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.file.vo.FileDetail;
import com.github.jfcloud.gene.file.vo.FileVo;
import com.github.jfcloud.gene.flow.dto.FlowDetailDto;
import com.github.jfcloud.gene.flow.dto.FlowPageDto;
import com.github.jfcloud.gene.flow.entity.FlowAudit;
import com.github.jfcloud.gene.flow.entity.FlowInfo;
import com.github.jfcloud.gene.flow.mapper.FlowInfoMapper;
import com.github.jfcloud.gene.flow.service.FlowAuditService;
import com.github.jfcloud.gene.flow.service.FlowFileVersionService;
import com.github.jfcloud.gene.flow.service.FlowInfoService;
import com.github.jfcloud.gene.flow.service.NotifyService;
import com.github.jfcloud.gene.flow.vo.FlowAuditVo;
import com.github.jfcloud.gene.flow.vo.FlowDetailVo;
import com.github.jfcloud.gene.flow.vo.FlowPageVo;
import com.github.jfcloud.gene.flow.vo.SynProjectVo;
import com.github.jfcloud.gene.form.dto.StrainCustomDto;
import com.github.jfcloud.gene.form.dto.StrainPurificationInfoDto;
import com.github.jfcloud.gene.form.entity.StrainCustomInfo;
import com.github.jfcloud.gene.form.entity.StrainPurificationInfo;
import com.github.jfcloud.gene.form.service.StrainCustomInfoService;
import com.github.jfcloud.gene.form.service.StrainPurificationInfoService;
import com.github.jfcloud.gene.sys.service.DBSystemPropertiesService;
import com.github.jfcloud.gene.util.WordDataService;
import com.github.jfcloud.gene.util.WordUtil;
import com.github.jfcloud.project.api.dto.LacmsProjectDTO;
import com.github.jfcloud.project.api.dto.ProjectFeignDTO;
import com.github.jfcloud.project.api.feign.RemoteProjectService;
import com.github.jfcloud.project.api.feign.SubProjectFeign;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Slf4j
@Service
@RequiredArgsConstructor
public class FlowInfoServiceImpl extends ServiceImpl<FlowInfoMapper, FlowInfo> implements FlowInfoService {

    private final UserIdNameCache userIdNameCache;
    private final StrainCustomInfoService customInfoService;
    private final StrainPurificationInfoService purificationInfoService;
    private final NotifyService notifyService;
    private final WordDataService wordDataService;
    private final FileInfoService fileInfoService;
    private final FlowFileVersionService fileVersionService;
    private final RemoteProjectService remoteProjectService;
    private final SubProjectFeign subProjectFeign;
    private final FlowAuditService flowAuditService;
    private final DBSystemPropertiesService systemPropertiesService;

    @Override
    public Page<FlowPageDto> getPage(FlowPageVo vo) {
        Page<FlowInfo> pageQuery = new Page<>(vo.getCurrent(), vo.getSize());

        //如果查询待提交，将驳回状态的一并查出
        if (vo.getStatusList().contains(GeneStatusEnum.DRAFT.getStatus())) {
            vo.getStatusList().add(GeneStatusEnum.REJECTED.getStatus());
        }

        LambdaQueryWrapper<FlowInfo> lqw = new LambdaQueryWrapper<>(FlowInfo.class)
                .like(StrUtil.isNotBlank(vo.getProjectName()), FlowInfo::getProjectName, vo.getProjectName())
                .like(StrUtil.isNotBlank(vo.getContractNo()), FlowInfo::getContractNo, vo.getContractNo())
                .like(StrUtil.isNotBlank(vo.getApprovalNo()), FlowInfo::getApprovalNo, vo.getApprovalNo())
                .eq(Objects.nonNull(vo.getProjectLeaderId()), FlowInfo::getProjectLeaderId, vo.getProjectLeaderId())
                .eq(Objects.nonNull(vo.getCreateBy()), FlowInfo::getCreateBy, vo.getCreateBy())
                .in(!vo.getStatusList().isEmpty(), FlowInfo::getStatus, vo.getStatusList())
                .eq(FlowInfo::getDeleted, WhetherEnum.NO.getCode())
                .orderByDesc(FlowInfo::getCreateTime);
        Page<FlowInfo> page = baseMapper.selectPageByScope(pageQuery, lqw, DataScope.of());
        if (page.getTotal() < 1) {
            return new Page<>();
        }

        List<FlowPageDto> records = BeanUtil.copyToList(page.getRecords(), FlowPageDto.class);

        records.forEach(item -> {
            //补充申请人姓名
            item.setCreateByName(userIdNameCache.getNicknameByUserId(Long.parseLong(item.getCreateBy())));
            //查询下载链接
            FileVo latestFile = fileVersionService.getLatestFile(item.getId());
            item.setFileUrl(latestFile.getUrl());
            item.setFilename(latestFile.getName());
            item.setStatusLabel(Objects.requireNonNull(GeneStatusEnum.getByStatus(item.getStatus())).getDescription());
        });

        Page<FlowPageDto> pageDto = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
        return pageDto.setRecords(records);
    }

    @Override
    public FlowDetailDto getDetail(Long id) {
        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");

        FlowDetailDto detail = BeanUtil.copyProperties(flowInfo, FlowDetailDto.class);
        detail.setGeneEditPlName(userIdNameCache.getNicknameByUserId(flowInfo.getGeneEditPlId()));
        detail.setProjectManageName(userIdNameCache.getNicknameByUserId(flowInfo.getProjectManageId()));
        detail.setStatusLabel(Objects.requireNonNull(GeneStatusEnum.getByStatus(detail.getStatus())).getDescription());

        StrainCustomDto customDto = customInfoService.getByFlowId(id);
        detail.setCustom(Optional.ofNullable(customDto).orElse(new StrainCustomDto()));
        StrainPurificationInfoDto purificationInfoDto = purificationInfoService.getByFlowId(id);
        detail.setPurification(Optional.ofNullable(purificationInfoDto).orElse(new StrainPurificationInfoDto()));

        //查询下载链接
        FileVo latestFile = fileVersionService.getLatestFile(id);
        detail.setFileUrl(latestFile.getUrl());
        detail.setFilename(latestFile.getName());

        return detail;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveFlow(FlowDetailVo vo) {
        log.info("保存流程 项目名称：{}", vo.getProjectName());
        FlowInfo flowInfo = BeanUtil.copyProperties(vo, FlowInfo.class);
        flowInfo.setId(CustomIdGenerator.nextId());

        boolean targetCheck = false;
        if (vo.getTarget().contains(GeneTargetEnum.CUSTOM.getCode())) {
            targetCheck = true;
            vo.getCustom().setFlowId(flowInfo.getId());
            customInfoService.saveForm(vo.getCustom());
        }

        if (vo.getTarget().contains(GeneTargetEnum.PURIFICATION.getCode())) {
            targetCheck = true;
            vo.getPurification().setFlowId(flowInfo.getId());
            purificationInfoService.saveForm(vo.getPurification());
        }

        Assert.isTrue(targetCheck, "target只能为custom purification");

        flowInfo.setStatus(GeneStatusEnum.DRAFT.getStatus());
        flowInfo.insert();

        geneWord(flowInfo.getId());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateFlow(Long id, FlowDetailVo vo) {
        log.info("编辑流程 项目名称：{}", vo.getProjectName());

        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");
        BeanUtil.copyProperties(vo, flowInfo);
        flowInfo.updateById();

        if (vo.getTarget().contains(GeneTargetEnum.CUSTOM.getCode())) {
            vo.getCustom().setFlowId(id);
            customInfoService.updateForm(vo.getCustom());
        } else {
            customInfoService.remove(new LambdaQueryWrapper<>(StrainCustomInfo.class).eq(StrainCustomInfo::getFlowId, id));
        }

        if (vo.getTarget().contains(GeneTargetEnum.PURIFICATION.getCode())) {
            vo.getPurification().setFlowId(id);
            purificationInfoService.updateForm(vo.getPurification());
        } else {
            purificationInfoService.remove(new LambdaQueryWrapper<>(StrainPurificationInfo.class).eq(StrainPurificationInfo::getFlowId, id));
        }

        geneWord(flowInfo.getId());
    }

    @Override
    public void submitFlow(Long id) {
        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");
        Assert.isTrue(GeneStatusEnum.SUBMIT_STATUS.contains(flowInfo.getStatus()), "流程状态错误");

        FlowInfo updateInfo = new FlowInfo().setId(id).setStatus(GeneStatusEnum.GENE_EDITING.getStatus());
        //如果是驳回重新提交，修改申请日期和申请人
        if (GeneStatusEnum.REJECTED.getStatus().equals(flowInfo.getStatus())) {
            log.info("流程 id={} 驳回后重新提交，更新申请日期和申请人", flowInfo.getId());
            updateInfo.setVersion(flowInfo.getVersion() + 1)
                    .setCreateBy(UserUtil.getUserId().toString())
                    .setCreateTime(new Date());
        }
        updateInfo.updateById();

        notifyService.notify(id);
    }

    @Override
    public void audit(Long id, FlowAuditVo vo) {
        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");
        Assert.isFalse(GeneStatusEnum.SUBMIT_STATUS.contains(flowInfo.getStatus()), "流程状态错误");

        String nextStatus = "";
        GeneStatusEnum statusEnum = Objects.requireNonNull(GeneStatusEnum.getByStatus(flowInfo.getStatus()));
        switch (statusEnum) {
            case GENE_EDITING:
                Assert.isTrue(UserUtil.getUserId().equals(flowInfo.getGeneEditPlId()), "抱歉，您不能审核本条记录");
                nextStatus = GeneStatusEnum.PROJECT_LEADER.getStatus();
                break;
            case PROJECT_LEADER:
                Assert.isTrue(UserUtil.getUserId().equals(flowInfo.getProjectLeaderId()), "抱歉，您不能审核本条记录");
                nextStatus = GeneStatusEnum.PROJECT_MANAGEMENT.getStatus();
                break;
            case PROJECT_MANAGEMENT:
                Assert.isTrue(UserUtil.getUserId().equals(flowInfo.getProjectManageId()), "抱歉，您不能审核本条记录");
                nextStatus = GeneStatusEnum.COMPLETED.getStatus();
                break;
            default:
                log.info("流程项目 ({}) 状态为{}", flowInfo.getProjectName(), statusEnum.getDescription());
                throw new IllegalArgumentException("流程状态错误");
        }

        //保存审批节点
        FlowAudit flowAudit = new FlowAudit()
                .setFlowId(id)
                .setFlowStatus(flowInfo.getStatus())
                .setAuditResult(vo.getAuditResult())
                .setRemarks(vo.getRemarks())
                .setCreateSign(userIdNameCache.getSignPic(UserUtil.getUserId()));
        if (StrUtil.isNotBlank(vo.getLeaderResult())) {
            JSONObject dataObj = new JSONObject().fluentPut("leaderResult", vo.getLeaderResult());
            flowAudit.setAdditionalData(dataObj.toJSONString());
        }
        flowAudit.insert();

        //审核不通过
        if (StrConstant.NO.equals(vo.getAuditResult())) {
            log.info("流程项目 ({}) 审核不通过", flowInfo.getProjectName());

            new FlowInfo()
                    .setId(id)
                    .setStatus(GeneStatusEnum.REJECTED.getStatus())
                    .updateById();

            geneWord(id);

            notifyService.notify(id);
            return;
        }

        Assert.isTrue(StrConstant.YES.equals(vo.getAuditResult()), "审核结果错误");
        if (GeneStatusEnum.PROJECT_MANAGEMENT.getStatus().equals(flowInfo.getStatus())) {
            Assert.isTrue(StrUtil.isNotBlank(vo.getApprovalNo()), "项目批准编号必填");
            //检查批准编号的唯一性
            long count = count(new LambdaQueryWrapper<>(FlowInfo.class)
                    .eq(FlowInfo::getApprovalNo, vo.getApprovalNo())
                    .eq(FlowInfo::getDeleted, WhetherEnum.NO.getCode()));
            Assert.isTrue(count < 1, "项目批准编号已存在");
        }

        //审核通过
        log.info("流程项目 ({}) 审核通过", flowInfo.getProjectName());

        flowInfo = new FlowInfo();
        //更新项目批准编号、修订号等
        BeanUtil.copyProperties(vo, flowInfo);
        flowInfo.setId(id)
                .setStatus(nextStatus)
                .updateById();

        geneWord(id);

        notifyService.notify(id);
    }

    @Override
    public void execute(Long id, SynProjectVo vo) {
        if (!systemPropertiesService.getBooleanValue("gene.sync.ky")) {
            throw new RuntimeException("基因定制-科研同步配置未开启，不进行同步");
        }

        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");
        Assert.isTrue(GeneStatusEnum.COMPLETED.getStatus().equals(flowInfo.getStatus()), "基因定制流程非完成状态");
        Assert.isNull(flowInfo.getKyId(), "基因定制流程已执行");

        if (Objects.isNull(vo.getApprovalType())) {
            String approvalType = systemPropertiesService.getValue("gene.approvalType");
            Assert.notEmpty(approvalType, "项目类型为空，无法同步");
            vo.setApprovalType(Long.parseLong(approvalType));
        }

        //推送数据到科研项目，并得到科研项目id
        ProjectFeignDTO projectFeignDTO = new ProjectFeignDTO();
        projectFeignDTO.setName(flowInfo.getProjectName());
        projectFeignDTO.setLeader(vo.getProjectLeader());
        projectFeignDTO.setProjectType(vo.getApprovalType());
        projectFeignDTO.setProjectCode(flowInfo.getApprovalNo());
        projectFeignDTO.setApplicantUserId(Long.parseLong(flowInfo.getCreateBy()));

        //推送申请人nickname
        String nickname = userIdNameCache.getNicknameByUserId(vo.getProjectLeader());
        projectFeignDTO.setLeaders(nickname);

        log.info("基因定制推送科研项目：flowId={}, param={}", id, JSON.toJSONString(projectFeignDTO));
        R<Long> longR = remoteProjectService.initProject(projectFeignDTO);
        if (!longR.isOk()) {
            log.error("projectId={}推送科研失败，{}", id, longR);
        }
        Assert.isTrue(longR.isOk(), "操作失败，科研" + longR);
        Long kyId = longR.getData();
        new FlowInfo().setId(id).setKyId(kyId).updateById();

        if (kyId != null && systemPropertiesService.getBooleanValue("sample.sync.animal")) {
            //同步动物中心，生成动物中心项目信息
            LacmsProjectDTO lacmsProjectDTO = new LacmsProjectDTO();
            lacmsProjectDTO.setKyId(kyId);
            lacmsProjectDTO.setProjectName(flowInfo.getProjectName());
            lacmsProjectDTO.setProjectLeader(vo.getProjectLeader());
            lacmsProjectDTO.setPl(flowInfo.getProjectLeaderName());
            lacmsProjectDTO.setEthicsCode(flowInfo.getApprovalNo());
            lacmsProjectDTO.setProjectType("基因定制");
            lacmsProjectDTO.setProjectNum(flowInfo.getApprovalNo());
            log.info("推送动物中心：flowId={}, param={}", id, JSON.toJSONString(lacmsProjectDTO));
            subProjectFeign.initProject(lacmsProjectDTO);
        } else {
            log.warn("基因定制-科研同步配置未开启，不进行同步");
        }

        //通知相关人
        String userIds = systemPropertiesService.getValue("gene.execute.id");
        if (StrUtil.isNotBlank(userIds)) {
            //钉钉通知
            if (systemPropertiesService.getBooleanValue("gene.execute.dingding.enable")) {
                List<MessageUserDTO> userList = Stream.of(userIds.split(","))
                        .filter(StrUtil::isNotBlank)
                        .map(x -> {
                            MessageUserDTO messageUserDTO = new MessageUserDTO();
                            messageUserDTO.setUserId(Long.parseLong(x));
                            return messageUserDTO;
                        })
                        .collect(Collectors.toList());
                String dingMsg = String.format("【基因定制/%s】【%s】已执行，请查看", flowInfo.getProjectName(), UserUtil.getNickName());
                notifyService.sendDingding("基因定制", dingMsg, userList);
            }
        }
    }

    /**
     * 生成word
     */
    @SneakyThrows
    public void geneWord(Long flowId) {
        FlowDetailDto detail = getDetail(flowId);
        FlowInfo flowInfo = getById(flowId);

        //文件名称：项目名称-负责人-申请日期-随机四位.docx
        String createDate = DateUtil.format(flowInfo.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN);
        String fileName = String.format("立项报告-动物%s申请表-%s-%s-%s-%s.docx",
                GeneTargetEnum.getNameByCode(flowInfo.getTarget()),
                StrUtil.nullToEmpty(detail.getProjectName()), StrUtil.nullToEmpty(detail.getProjectLeaderName()), createDate,
                RandomUtil.randomString(RandomUtil.BASE_CHAR_NUMBER_LOWER, 4));
        log.info("生成word 项目名称：{} 文件名称：{}", detail.getProjectName(), fileName);

        JSONObject dataMap = JSON.parseObject(JSON.toJSONString(detail));
        wordDataService.fillDocCreator(dataMap);

        //待提交状态下不显示申请人和申请日期
        if (!GeneStatusEnum.SUBMIT_STATUS.contains(flowInfo.getStatus())) {
            dataMap.put("createDateStr", createDate);
            String sign = userIdNameCache.getSignBase64(Long.valueOf(flowInfo.getCreateBy()));
            if (StrUtil.isNotBlank(sign)) {
                dataMap.put("createBySign", sign);
            }
        }

        //遗传背景说明
        if (detail.getTarget().contains(GeneTargetEnum.PURIFICATION.getCode())) {
            JSONObject noteObj = new JSONObject();
            String geneticBackgroundNote = detail.getPurification().getGeneticBackgroundNote();
            if (StrUtil.isNotBlank(geneticBackgroundNote)) {
                noteObj = JSON.parseObject(geneticBackgroundNote);
            }
            dataMap.getJSONObject("purification").put("geneticBackgroundNote", noteObj);
        }

        dataMap.put("approvalNo", WordUtil.padString(flowInfo.getApprovalNo(), 14));
        dataMap.put("revisionNo", WordUtil.padString(flowInfo.getRevisionNo(), 14));
        dataMap.put("receiptDate", WordUtil.padString(flowInfo.getReceiptDate(), 12));
        dataMap.put("approvalDate", WordUtil.padString(flowInfo.getApprovalDate(), 12));
        dataMap.put("animalEnrollment", WordUtil.padString(flowInfo.getAnimalEnrollment(), 12));
        dataMap.put("finalSampling", WordUtil.padString(flowInfo.getFinalSampling(), 12));

        //查询审核记录
        List<FlowAudit> flowAudits = flowAuditService.auditList(flowId, null, true);

        //项目管理部审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.PROJECT_MANAGEMENT.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("projectManagementDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    String base64 = wordDataService.getBase64(audit.getCreateSign());
                    if (StrUtil.isNotBlank(base64)) {
                        dataMap.put("projectManagementSign", base64);
                    }
                });
        //项目负责人审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.PROJECT_LEADER.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("projectLeaderDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    String base64 = wordDataService.getBase64(audit.getCreateSign());
                    if (StrUtil.isNotBlank(base64)) {
                        dataMap.put("projectLeaderSign", base64);
                    }
                    if (StrUtil.isNotBlank(audit.getAdditionalData())) {
                        JSONObject additionObj = JSON.parseObject(audit.getAdditionalData());
                        dataMap.put("leaderResult", StrConstant.YES.equals(additionObj.getOrDefault("leaderResult", "")));
                    }
                });
        //基因编辑部门审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.GENE_EDITING.getStatus().equals(audit.getFlowStatus()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("geneEditingResult", StrConstant.YES.equals(audit.getAuditResult()));
                    dataMap.put("geneEditingRemarks", audit.getRemarks());
                });


        String modelName = "geneCustom.ftl";
        fileInfoService.uploadFileWithFileName(modelName, fileName, flowInfo.getId(), flowInfo.getStatus(), dataMap);
    }

    @Override
    public void copyFlow(Long id) {
        FlowDetailDto detail = getDetail(id);
        FlowDetailVo vo = BeanUtil.copyProperties(detail, FlowDetailVo.class);
        vo.setProjectName(vo.getProjectName() + "(1)");
        saveFlow(vo);
    }

    @Override
    public List<FileDetail> getFile(Long id) {
        FileVo latestFile = fileVersionService.getLatestFile(id);
        if (StrUtil.isBlank(latestFile.getUrl())) {
            return Collections.emptyList();
        }

        FileDetail fileDetail = fileInfoService.getFileSize(latestFile.getUrl())
                .setUrl(latestFile.getUrl())
                .setFilename(latestFile.getName());
        return Collections.singletonList(fileDetail);
    }
}
