package com.github.jfcloud.gene.form.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.form.dto.StrainCustomDto;
import com.github.jfcloud.gene.form.entity.StrainCustomDetail;
import com.github.jfcloud.gene.form.entity.StrainCustomInfo;
import com.github.jfcloud.gene.form.mapper.StrainCustomInfoMapper;
import com.github.jfcloud.gene.form.service.StrainCustomDetailService;
import com.github.jfcloud.gene.form.service.StrainCustomInfoService;
import com.github.jfcloud.gene.form.vo.StrainCustomDetailVo;
import com.github.jfcloud.gene.form.vo.StrainCustomVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class StrainCustomInfoServiceImpl extends ServiceImpl<StrainCustomInfoMapper, StrainCustomInfo> implements StrainCustomInfoService {

    private final StrainCustomDetailService strainCustomDetailService;

    @Override
    public void saveForm(StrainCustomVo vo) {
        StrainCustomInfo customInfo = BeanUtil.copyProperties(vo, StrainCustomInfo.class);
        customInfo.setId(CustomIdGenerator.nextId());
        customInfo.insert();

        //保存总表
        if (!vo.getDetailList().isEmpty()) {
            List<StrainCustomDetail> detailList = vo.getDetailList().stream()
                    .map(x -> {
                        StrainCustomDetail detail = BeanUtil.copyProperties(x, StrainCustomDetail.class);
                        detail.setStrainCustomBasicId(customInfo.getId());
                        detail.setId(CustomIdGenerator.nextId());
                        return detail;
                    })
                    .collect(Collectors.toList());
            strainCustomDetailService.saveBatch(detailList);
        }
    }

    @Override
    public void updateForm(StrainCustomVo vo) {
        StrainCustomInfo info = getOne(new LambdaQueryWrapper<>(StrainCustomInfo.class)
                .eq(StrainCustomInfo::getFlowId, vo.getFlowId())
                .eq(StrainCustomInfo::getDeleted, WhetherEnum.NO.getCode()));
        if (info == null) {
            saveForm(vo);
            return;
        }

        BeanUtil.copyProperties(vo, info);
        info.updateById();

        //先删除总表，再保存总表
        strainCustomDetailService.remove(new LambdaQueryWrapper<StrainCustomDetail>().eq(StrainCustomDetail::getStrainCustomBasicId, info.getId()));
        if (!vo.getDetailList().isEmpty()) {
            List<StrainCustomDetail> detailList = vo.getDetailList().stream()
                    .map(x -> {
                        StrainCustomDetail detail = BeanUtil.copyProperties(x, StrainCustomDetail.class);
                        detail.setStrainCustomBasicId(info.getId());
                        detail.setId(CustomIdGenerator.nextId());
                        return detail;
                    })
                    .collect(Collectors.toList());
            strainCustomDetailService.saveBatch(detailList);
        }
    }

    @Override
    public StrainCustomDto getByFlowId(Long flowId) {
        StrainCustomInfo info = getOne(new LambdaQueryWrapper<StrainCustomInfo>()
                .eq(StrainCustomInfo::getFlowId, flowId)
                .eq(StrainCustomInfo::getDeleted, WhetherEnum.NO.getCode()));
        if (info == null) {
            return null;
        }

        StrainCustomDto dto = BeanUtil.copyProperties(info, StrainCustomDto.class);
        List<StrainCustomDetail> detailList = strainCustomDetailService.list(new LambdaQueryWrapper<StrainCustomDetail>()
                .eq(StrainCustomDetail::getStrainCustomBasicId, info.getId())
                .eq(StrainCustomDetail::getDeleted, WhetherEnum.NO.getCode())
                .orderByAsc(StrainCustomDetail::getId));
        dto.setDetailList(BeanUtil.copyToList(detailList, StrainCustomDetailVo.class));
        return dto;

    }
}
