package com.github.jfcloud.gene.form.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.file.entity.FileInfo;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.form.dto.StrainPurificationInfoDto;
import com.github.jfcloud.gene.form.entity.AnimalDemand;
import com.github.jfcloud.gene.form.entity.CageDemand;
import com.github.jfcloud.gene.form.entity.StrainPurificationInfo;
import com.github.jfcloud.gene.form.mapper.StrainPurificationInfoMapper;
import com.github.jfcloud.gene.form.service.AnimalDemandService;
import com.github.jfcloud.gene.form.service.CageDemandService;
import com.github.jfcloud.gene.form.service.StrainPurificationInfoService;
import com.github.jfcloud.gene.form.vo.AnimalDemandVo;
import com.github.jfcloud.gene.form.vo.CageDemandVo;
import com.github.jfcloud.gene.form.vo.StrainPurificationInfoVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class StrainPurificationInfoServiceImpl extends ServiceImpl<StrainPurificationInfoMapper, StrainPurificationInfo> implements StrainPurificationInfoService {

    private final CageDemandService cageDemandService;

    private final AnimalDemandService animalDemandService;

    private final FileInfoService fileInfoService;

    @Override
    public void saveForm(StrainPurificationInfoVo vo) {
        StrainPurificationInfo purificationInfo = BeanUtil.copyProperties(vo, StrainPurificationInfo.class);
        purificationInfo.setId(CustomIdGenerator.nextId());
        if (vo.getGeneticBackgroundNoteObj() != null) {
            purificationInfo.setGeneticBackgroundNote(vo.getGeneticBackgroundNoteObj().toJSONString());
        }
        purificationInfo.insert();

        saveDetailList(purificationInfo.getId(), vo);
    }

    private void saveDetailList(Long id, StrainPurificationInfoVo vo) {
        //保存笼位需求
        if (!vo.getCageDemands().isEmpty()) {
            List<CageDemand> cageDemandList = vo.getCageDemands().stream()
                    .map(x -> {
                        CageDemand cageDemand = BeanUtil.copyProperties(x, CageDemand.class);
                        cageDemand.setPurificationId(id);
                        cageDemand.setId(CustomIdGenerator.nextId());
                        return cageDemand;
                    })
                    .collect(Collectors.toList());
            cageDemandService.saveBatch(cageDemandList);
        }

        //保存动物需求
        if (!vo.getAnimalDemands().isEmpty()) {
            List<AnimalDemand> cageDemandList = vo.getAnimalDemands().stream()
                    .map(x -> {
                        AnimalDemand animalDemand = BeanUtil.copyProperties(x, AnimalDemand.class);
                        animalDemand.setPurificationId(id);
                        animalDemand.setId(CustomIdGenerator.nextId());
                        return animalDemand;
                    })
                    .collect(Collectors.toList());
            animalDemandService.saveBatch(cageDemandList);
        }


        //保存附件
        if (!vo.getTestingFiles().isEmpty()) {
            List<FileInfo> fileList = vo.getTestingFiles().stream()
                    .map(x -> {
                        FileInfo fileInfo = BeanUtil.copyProperties(x, FileInfo.class);
                        fileInfo.setRelateId(id);
                        fileInfo.setType(FileNameUtil.getSuffix(x.getName()));
                        return fileInfo;
                    })
                    .collect(Collectors.toList());
            fileInfoService.saveBatch(fileList);
        }
    }

    @Override
    public void updateForm(StrainPurificationInfoVo vo) {
        StrainPurificationInfo info = getOne(new LambdaQueryWrapper<>(StrainPurificationInfo.class)
                .eq(StrainPurificationInfo::getFlowId, vo.getFlowId())
                .eq(StrainPurificationInfo::getDeleted, WhetherEnum.NO.getCode()));

        if (info == null) {
            saveForm(vo);
            return;
        }

        BeanUtil.copyProperties(vo, info);
        if (vo.getGeneticBackgroundNoteObj() != null) {
            info.setGeneticBackgroundNote(vo.getGeneticBackgroundNoteObj().toJSONString());
        }
        info.updateById();

        //先删除附件等，再进行保存
        cageDemandService.remove(new LambdaQueryWrapper<>(CageDemand.class).eq(CageDemand::getPurificationId, info.getId()));
        animalDemandService.remove(new LambdaQueryWrapper<>(AnimalDemand.class).eq(AnimalDemand::getPurificationId, info.getId()));
        fileInfoService.remove(new LambdaQueryWrapper<>(FileInfo.class).eq(FileInfo::getRelateId, info.getId()));

        saveDetailList(info.getId(), vo);
    }

    @Override
    public StrainPurificationInfoDto getByFlowId(Long id) {
        StrainPurificationInfo info = getOne(new LambdaQueryWrapper<StrainPurificationInfo>()
                .eq(StrainPurificationInfo::getFlowId, id)
                .eq(StrainPurificationInfo::getDeleted, WhetherEnum.NO.getCode()));
        if (info == null) {
            return null;
        }

        StrainPurificationInfoDto dto = BeanUtil.copyProperties(info, StrainPurificationInfoDto.class);
        if (StrUtil.isNotBlank(info.getGeneticBackgroundNote())) {
            dto.setGeneticBackgroundNoteObj(JSONObject.parseObject(info.getGeneticBackgroundNote()));
            dto.setGeneticBackgroundNote(null);
        }
        //动物需求
        List<AnimalDemand> animalDemandList = animalDemandService.list(new LambdaQueryWrapper<AnimalDemand>()
                .eq(AnimalDemand::getPurificationId, info.getId())
                .eq(AnimalDemand::getDeleted, WhetherEnum.NO.getCode()));
        dto.setAnimalDemands(BeanUtil.copyToList(animalDemandList, AnimalDemandVo.class));
        //笼位需求
        List<CageDemand> cageDemands = cageDemandService.list(new LambdaQueryWrapper<CageDemand>()
                .eq(CageDemand::getPurificationId, info.getId())
                .eq(CageDemand::getDeleted, WhetherEnum.NO.getCode()));
        dto.setCageDemands(BeanUtil.copyToList(cageDemands, CageDemandVo.class));
        //附件
        dto.setTestingFiles(fileInfoService.listByRelateId(info.getId()));
        return dto;
    }
}
