package com.github.jfcloud.gene.form.vo;

import com.alibaba.fastjson.JSONObject;
import com.github.jfcloud.gene.file.vo.FileVo;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.util.ArrayList;
import java.util.List;

@Data
@Schema(description = "品系净化扩繁信息实体类")
public class StrainPurificationInfoVo {

    @Schema(hidden = true)
    private Long flowId;

    /**
     * 品系名称
     */
    @Schema(description = "品系名称")
    private String strainName;
    /**
     * 动物饲养地
     */
    @Schema(description = "动物饲养地")
    private String breedingLocation;
    /**
     * 获得来源
     */
    @Schema(description = "获得来源", example = "模式动物中心提供")
    private String source;
    /**
     * 引种来源官方网站链接
     */
    @Schema(description = "引种来源官方网站链接")
    private String sourceLink;
    /**
     * 自行提供外部动物信息
     */
    @Schema(description = "自行提供外部动物信息")
    private String externalAnimalInfo;
    /**
     * 动物出生信息、基因型、鉴定策略
     */
    @Schema(description = "动物出生信息、基因型、鉴定策略")
    private String birthGeneIdentification;
    /**
     * 实验安排
     */
    @Schema(description = "实验安排", example = "仅净化保种,扩繁,精子冷冻")
    private String experimentalArrangement;
    /**
     * 遗传背景
     */
    @Schema(description = "遗传背景", example = "KO,CKO")
    private String geneticBackground;

    @Schema(description = "遗传背景说明")
    private JSONObject geneticBackgroundNoteObj = new JSONObject();
    /**
     * 遗传背景（其他）
     */
    @Schema(description = "遗传背景（其他）")
    private String geneticBackgroundOther;
    /**
     * 雌性繁育方式
     */
    @Schema(description = "雌性繁育方式", example = "杂合")
    private String femaleBreedingMethod;
    /**
     * 雄性繁育方式
     */
    @Schema(description = "雄性繁育方式", example = "纯合")
    private String maleBreedingMethod;
    /**
     * 有无明显表型特征
     */
    @Schema(description = "有无明显表型特征")
    private String phenotype;
    /**
     * 表型描述
     */
    @Schema(description = "表型特征描述，表型特征选[是]显示")
    private String phenotypeDescription;
    /**
     * 是否做过微生物检测
     */
    @Schema(description = "是否做过微生物检测")
    private String microbialTesting;
    /**
     * 检测细节
     */
    @Schema(description = "检测细节，微生物检测选[是]显示")
    private String testingDetails;
    /**
     * 检测附件
     */
    @Schema(description = "检测附件")
    private List<FileVo> testingFiles = new ArrayList<>();

    @Schema(description = "动物需求")
    private List<AnimalDemandVo> animalDemands = new ArrayList<>();

    @Schema(description = "笼位需求")
    private List<CageDemandVo> cageDemands = new ArrayList<>();

    /**
     * 基因鉴定
     */
    @Schema(description = "基因鉴定", example = "申请方自行鉴定")
    private String geneTesting;
    /**
     * 实验后期安排
     */
    @Schema(description = "实验后期安排", example = "净化后扩繁做实验")
    private String experimentalPostArrangement;
    /**
     * 冻存服务需求
     */
    @Schema(description = "冻结服务需求", example = "冻存精子/冻存胚胎")
    private String cryopreservationService;
    /**
     * 冻存周期
     */
    @Schema(description = "冻结周期", example = "3年")
    private String cryopreservationPeriod;
    /**
     * 冻存数量（如精子麦管数、胚胎数）
     */
    @Schema(description = "冻结数量（如精子麦管数、胚胎数）")
    private Integer cryopreservationCount;
}
