package com.github.jfcloud.gene.sample.controller;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.Pair;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.jfcloud.common.annotation.WebApiLog;
import com.github.jfcloud.common.core.util.R;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.file.vo.FileDetail;
import com.github.jfcloud.gene.flow.service.FlowInfoService;
import com.github.jfcloud.gene.flow.vo.SynProjectVo;
import com.github.jfcloud.gene.sample.dto.SampleAuditDto;
import com.github.jfcloud.gene.sample.dto.SampleDetailDto;
import com.github.jfcloud.gene.sample.dto.SamplePageDto;
import com.github.jfcloud.gene.sample.entity.SampleInfo;
import com.github.jfcloud.gene.sample.service.SampleInfoService;
import com.github.jfcloud.gene.sample.vo.SampleAuditVo;
import com.github.jfcloud.gene.sample.vo.SamplePageVo;
import com.github.jfcloud.gene.sample.vo.SampleSubmitVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Tag(name = "样本送检流程")
@RestController
@RequestMapping("/sample")
@RequiredArgsConstructor
public class SampleController {

    private final SampleInfoService sampleInfoService;
    private final FlowInfoService flowInfoService;

    @WebApiLog
    @Operation(summary = "列表查询")
    @GetMapping("/page")
    public R<Page<SamplePageDto>> page(SamplePageVo vo) {
        return R.ok(sampleInfoService.getPage(vo));
    }

    @Operation(summary = "详情")
    @GetMapping("/{id}")
    public R<SampleDetailDto> detail(@PathVariable Long id) {
        return R.ok(sampleInfoService.getDetail(id));
    }

    @Operation(summary = "状态列表")
    @GetMapping("/statusList")
    public R<List<Pair<String, String>>> statusList() {
        return R.ok(GeneStatusEnum.getSampleStatusList());
    }

    @WebApiLog
    @Operation(summary = "新增")
    @PostMapping
    public R save(@Valid @RequestBody SampleSubmitVo vo) {
        sampleInfoService.saveSample(vo);
        return R.ok();
    }

    @WebApiLog
    @Operation(summary = "编辑")
    @PostMapping("/edit/{id}")
    public R update(@PathVariable Long id, @Valid @RequestBody SampleSubmitVo vo) {
        sampleInfoService.updateSample(id, vo);
        return R.ok();
    }

    @Operation(summary = "删除")
    @PostMapping("/del/{id}")
    public R delete(@PathVariable Long id) {
        sampleInfoService.removeById(id);
        return R.ok();
    }

    @Operation(summary = "提交")
    @PostMapping("/submit/{id}")
    public R submit(@PathVariable Long id) {
        sampleInfoService.submit(id);
        return R.ok();
    }

    @Operation(summary = "复制")
    @PostMapping("/copy/{id}")
    public R copy(@PathVariable Long id) {
        sampleInfoService.copySample(id);
        return R.ok();
    }

    @WebApiLog
    @Operation(summary = "审核")
    @PostMapping("/audit/{id}")
    public R audit(@PathVariable Long id, @Valid @RequestBody SampleAuditVo vo) {
        sampleInfoService.audit(id, vo);
        return R.ok();
    }

    @Operation(summary = "审核记录")
    @GetMapping("/audit/{id}")
    public R<List<SampleAuditDto>> auditRecord(@PathVariable Long id) {
        return R.ok(sampleInfoService.getAuditRecords(id));
    }

    @WebApiLog
    @Operation(summary = "执行")
    @PostMapping("/execute/{id}")
    public R execute(@PathVariable Long id, @RequestBody @Valid SynProjectVo vo) {
        sampleInfoService.execute(id, vo);
        return R.ok();
    }

    @Operation(summary = "生成文档", hidden = true)
    @PostMapping("/generate/{id}")
    public R generate(@PathVariable Long id) {
        sampleInfoService.generate(id);
        return R.ok();
    }

    @Operation(summary = "查询流程附件")
    @GetMapping("/file")
    public R<List<FileDetail>> getProjectFile(@RequestParam(required = false) Long id, @RequestParam(required = false) Long kyId) {
        Assert.isTrue(id != null || kyId != null, "参数错误");
        SampleInfo sampleInfo = sampleInfoService.getOne(new LambdaQueryWrapper<>(SampleInfo.class)
                .select(SampleInfo::getId)
                .eq(id != null, SampleInfo::getId, id)
                .eq(kyId != null, SampleInfo::getKyId, kyId)
                .eq(SampleInfo::getDeleted, WhetherEnum.NO.getCode()));
        if (sampleInfo == null) {
            return R.failed("找不到对应的流程");
        }
        return R.ok(flowInfoService.getFile(sampleInfo.getId()));
    }
}
