package com.github.jfcloud.gene.sample.enums;

import cn.hutool.core.util.StrUtil;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 病原样本类型
 */
public enum PathogenSampleTypeEnum {
    PIG("新入驻猪送检"),
    RODENT("啮齿类检测"),
    MONKEY("猴检测"),
    DOG("犬检测"),
    CAT("猫检测"),
    GOAT("羊检测"),
    MATERIAL("物料环境检测");

    private final String description;

    PathogenSampleTypeEnum(String description) {
        this.description = description;
    }

    public static PathogenSampleTypeEnum resolve(String type) {
        for (PathogenSampleTypeEnum value : values()) {
            if (value.getDescription().equals(type)) {
                return value;
            }
        }
        return null;
    }

    public static List<String> resolveList(String type) {
        if (StrUtil.isBlank(type)) {
            return Collections.emptyList();
        }
        List<String> list = new ArrayList<>();
        for (String s : type.split(",")) {
            PathogenSampleTypeEnum item = resolve(s);
            if (item != null) {
                list.add(item.name().toLowerCase());
            }
        }

        return list;
    }

    public String getDescription() {
        return description;
    }
}
