package com.github.jfcloud.gene.sample.service.biz;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.admin.api.sys.dto.message.MessageUserDTO;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.constant.StrConstant;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.common.util.UserUtil;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.flow.entity.FlowAudit;
import com.github.jfcloud.gene.flow.service.FlowAuditService;
import com.github.jfcloud.gene.flow.service.NotifyService;
import com.github.jfcloud.gene.sample.entity.*;
import com.github.jfcloud.gene.sample.enums.SampleTypeEnum;
import com.github.jfcloud.gene.sample.mapper.SampleAnimalImagingMapper;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.service.*;
import com.github.jfcloud.gene.sample.vo.*;
import com.github.jfcloud.gene.sys.service.DBSystemPropertiesService;
import com.github.jfcloud.gene.util.WordDataService;
import com.github.jfcloud.gene.util.WordUtil;
import com.github.jfcloud.rich.vo.HtmlParseVo;
import com.github.jfcloud.rich.word.WordNodeFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class SampleAnimalImagingServiceImpl extends ServiceImpl<SampleAnimalImagingMapper, SampleAnimalImaging> implements SampleAnimalImagingService, SampleEditService {

    private final SampleInfoMapper sampleInfoMapper;
    private final LiveImagingDetailService liveImagingDetailService;
    private final MicroCtDetailService microCtDetailService;
    private final UltrasoundDetailService ultrasoundDetailService;
    private final WordDataService wordDataService;
    private final UserIdNameCache userIdNameCache;
    private final FlowAuditService flowAuditService;
    private final FileInfoService fileInfoService;
    private final DBSystemPropertiesService systemPropertiesService;
    private final NotifyService notifyService;

    @Override
    public void removeBySampleId(Long sampleId) {
        List<SampleAnimalImaging> imagings = list(new LambdaQueryWrapper<>(SampleAnimalImaging.class)
                .select(SampleAnimalImaging::getId)
                .eq(SampleAnimalImaging::getSampleId, sampleId));
        if (imagings.isEmpty()) {
            return;
        }

        Set<Long> imgIdSet = imagings.stream().map(SampleAnimalImaging::getId).collect(Collectors.toSet());
        removeBatchByIds(imgIdSet);
        liveImagingDetailService.remove(new LambdaQueryWrapper<>(LiveImagingDetail.class).in(LiveImagingDetail::getImagingId, imgIdSet));
        microCtDetailService.remove(new LambdaQueryWrapper<>(MicroCtDetail.class).in(MicroCtDetail::getImagingId, imgIdSet));
        ultrasoundDetailService.remove(new LambdaQueryWrapper<>(UltrasoundDetail.class).in(UltrasoundDetail::getImagingId, imgIdSet));
    }

    @Override
    public void save(Long sampleId, SampleSubmitVo vo) {
        SampleAnimalImagingVo animalImaging = vo.getAnimalImage();
        Assert.notNull(animalImaging, "动物影像不能为空");
        removeBySampleId(sampleId);

        SampleAnimalImaging sampleAnimalImaging = BeanUtil.copyProperties(animalImaging, SampleAnimalImaging.class);
        sampleAnimalImaging.setId(CustomIdGenerator.nextId());
        sampleAnimalImaging.setSampleId(sampleId);
        sampleAnimalImaging.insert();

        // 保存活体成像信息
        LiveImagingVo liveImaging = animalImaging.getLiveImaging();
        if (Objects.isNull(animalImaging.getRequirements())) {
            animalImaging.setRequirements("");
        }
        if (animalImaging.getRequirements().contains("活体成像") && liveImaging != null) {
            LiveImagingDetail liveImagingDetail = BeanUtil.copyProperties(liveImaging, LiveImagingDetail.class);
            liveImagingDetail.setImagingId(sampleAnimalImaging.getId());
            liveImagingDetail.insert();
        }

        // 保存Micro-CT信息
        MicroCtVo microCt = animalImaging.getMicroCt();
        if (animalImaging.getRequirements().contains("Micro") && microCt != null) {
            MicroCtDetail microCtDetail = BeanUtil.copyProperties(microCt, MicroCtDetail.class);
            microCtDetail.setImagingId(sampleAnimalImaging.getId());
            microCtDetail.insert();
        }

        // 保存超声信息
        UltrasoundVo ultrasound = animalImaging.getUltrasound();
        if (animalImaging.getRequirements().contains("超声") && ultrasound != null) {
            UltrasoundDetail ultrasoundDetail = BeanUtil.copyProperties(ultrasound, UltrasoundDetail.class);
            ultrasoundDetail.setImagingId(sampleAnimalImaging.getId());
            ultrasoundDetail.insert();
        }

        log.info("保存动物影像成功");
        generate(sampleId);
    }

    @Override
    public void generate(Long sampleId) {
        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);
        SampleAnimalImagingVo animalImagingVo = getDetail(sampleId);

        //文件名称：项目名称-申请人-申请日期-随机四位.docx
        String applyDate = DateUtil.format(sampleInfo.getApplyTime(), DatePattern.CHINESE_DATE_PATTERN);
        String fileName = String.format("动物影像检测分析申请表-%s-%s-%s-%s.docx",
                StrUtil.nullToEmpty(sampleInfo.getProjectName()), StrUtil.nullToEmpty(sampleInfo.getApplicantName()), applyDate,
                RandomUtil.randomString(RandomUtil.BASE_CHAR_NUMBER_LOWER, 4));
        log.info("生成word 项目名称：{} 文件名称：{}", sampleInfo.getProjectName(), fileName);

        JSONObject dataMap = new JSONObject();
        wordDataService.fillDocCreator(dataMap);
        JSONObject sampleInfoObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(sampleInfo, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("sampleInfo", sampleInfoObj);

        //富文本图片
        JSONArray images = new JSONArray();
        dataMap.put("images", images);

        //需求说明，避免Micro-CT显像与CT显像混合
        animalImagingVo.setRequirements(animalImagingVo.getRequirements().replace("Micro-CT显像", "Micro-CT"));
        //超声-心超数据，避免LVAWd与LVAW混合
        UltrasoundVo ultrasound = animalImagingVo.getUltrasound();
        if (StrUtil.isNotBlank(ultrasound.getHeartData())) {
            ultrasound.setHeartData(ultrasound.getHeartData()
                    .replace("LVAWd", "AWd")
                    .replace("e'/a'", "e/a"));
        }
        //活体成像-具体需求
        HtmlParseVo srParseVo = WordNodeFactory.html2xml(animalImagingVo.getLiveImaging().getSpecificRequirements());
        animalImagingVo.getLiveImaging().setSpecificRequirements(srParseVo.getContent());
        images.addAll(srParseVo.getImageCache());
        //超声-图片要求
        HtmlParseVo imgParseVo = WordNodeFactory.html2xml(ultrasound.getImageRequirements());
        ultrasound.setImageRequirements(imgParseVo.getContent());
        images.addAll(imgParseVo.getImageCache());
        //MicroCT-图片要求
        HtmlParseVo ctImgParseVo = WordNodeFactory.html2xml(animalImagingVo.getMicroCt().getImageRequirements());
        animalImagingVo.getMicroCt().setImageRequirements(ctImgParseVo.getContent());
        images.addAll(ctImgParseVo.getImageCache());
        //建模信息
        HtmlParseVo modelImgParseVo = WordNodeFactory.html2xml(animalImagingVo.getModelInfo());
        animalImagingVo.setModelInfo(modelImgParseVo.getContent());
        images.addAll(modelImgParseVo.getImageCache());
        //预期结果
        HtmlParseVo exParseVo = WordNodeFactory.html2xml(animalImagingVo.getExpectedResult());
        animalImagingVo.setExpectedResult(exParseVo.getContent());
        images.addAll(exParseVo.getImageCache());
        //前期论文
        HtmlParseVo ppParseVo = WordNodeFactory.html2xml(animalImagingVo.getPriorPublications());
        animalImagingVo.setPriorPublications(ppParseVo.getContent());
        images.addAll(ppParseVo.getImageCache());

        JSONObject animalImagingObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(animalImagingVo, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("animalImage", animalImagingObj);

        //检测人员
        if (StrUtil.isNotEmpty(sampleInfo.getInspector())) {
            String inspectorName = JSON.parseArray(sampleInfo.getInspector())
                    .stream()
                    .map(item -> {
                        JSONObject obj = (JSONObject) item;
                        return obj.getString("name");
                    })
                    .collect(Collectors.joining("、"));
            sampleInfoObj.put("inspectorName", inspectorName);
        }

        //申请时间
        String applyDateStr = DateUtil.format(sampleInfo.getApplyTime(), DatePattern.NORM_DATE_PATTERN);
        String[] applyDateSplit = applyDateStr.split("-");
        sampleInfoObj.put("applyTime", applyDateStr);
        sampleInfoObj.put("applyYear", applyDateSplit[0]);
        sampleInfoObj.put("applyMonth", applyDateSplit[1]);
        sampleInfoObj.put("applyDay", applyDateSplit[2]);

        //待提交状态下不显示申请人和申请日期
        if (!GeneStatusEnum.SUBMIT_STATUS.contains(sampleInfo.getStatus())) {
            dataMap.put("createDateStr", DateUtil.format(sampleInfo.getApplyTime(), DatePattern.CHINESE_DATE_PATTERN));
            String sign = userIdNameCache.getSignBase64(sampleInfo.getApplicantId());
            if (StrUtil.isNotBlank(sign)) {
                dataMap.put("createBySign", sign);
            }
        }

        //查询审核记录
        List<FlowAudit> flowAudits = flowAuditService.auditList(sampleId,"sample." + sampleInfo.getType(), true);
        //项目管理部审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.PROJECT_MANAGEMENT.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("projectManagementDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    String base64 = wordDataService.getBase64(audit.getCreateSign());
                    if (StrUtil.isNotBlank(base64)) {
                        dataMap.put("projectManagementSign", base64);
                    }
                });
        //部门负责人审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.DEPART_LEADER.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("departLeaderDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    String base64 = wordDataService.getBase64(audit.getCreateSign());
                    if (StrUtil.isNotBlank(base64)) {
                        dataMap.put("departLeaderSign", base64);
                    }
                });

        //项目管理部填写字段
        sampleInfoObj.put("approvalNo", WordUtil.padString(sampleInfo.getApprovalNo(), 14));
        sampleInfoObj.put("revisionNo", WordUtil.padString(sampleInfo.getRevisionNo(), 14));
        sampleInfoObj.put("receiptDate", WordUtil.padString(sampleInfo.getReceiptDate(), 8));
        sampleInfoObj.put("approvalDate", WordUtil.padString(sampleInfo.getApprovalDate(), 8));
        sampleInfoObj.put("expireDate", WordUtil.padString(sampleInfo.getExpireDate(), 8));

        String modelName = "animalImaging.ftl";
        fileInfoService.uploadFileWithFileName(modelName, fileName, sampleInfo.getId(), sampleInfo.getStatus(), dataMap);
    }

    @Override
    public void afterExecute(Long sampleId) {
        //钉钉消息通知申请人、检测人员
        if (!systemPropertiesService.getBooleanValue("sample.afterExecute.dingding.enable")) {
            return;
        }

        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);

        String projectType = "样本送检/" + Objects.requireNonNull(SampleTypeEnum.resolve(sampleInfo.getType())).getLabel();
        if (StrUtil.isNotBlank(sampleInfo.getProjectNo())) {
            projectType += "/" + sampleInfo.getProjectNo();
        }
        if (StrUtil.isNotBlank(sampleInfo.getProjectName())) {
            projectType += "/" + sampleInfo.getProjectName();
        }
        String dingMsg = String.format("【%s】【%s】已执行项目", projectType, UserUtil.getNickName());

        Set<Long> userIds = new HashSet<>();
        //申请人
        userIds.add(sampleInfo.getApplicantId());
        //检测人员
        if (StrUtil.isNotBlank(sampleInfo.getInspector())) {
            JSON.parseArray(sampleInfo.getInspector()).forEach(item -> {
                JSONObject obj = (JSONObject) item;
                if (obj.containsKey("id")) {
                    userIds.add(obj.getLong("id"));
                }
            });
        }

        List<MessageUserDTO> users = userIds.stream()
                .map(x -> {
                    MessageUserDTO messageUserDTO = new MessageUserDTO();
                    messageUserDTO.setUserId(x);
                    return messageUserDTO;
                })
                .collect(Collectors.toList());
        notifyService.sendDingding("样本送检", dingMsg, users);
    }

    @Override
    public SampleAnimalImagingVo getDetail(Long sampleId) {
        SampleAnimalImaging animalImaging = getOne(new LambdaQueryWrapper<>(SampleAnimalImaging.class)
                .eq(SampleAnimalImaging::getSampleId, sampleId)
                .eq(SampleAnimalImaging::getDeleted, WhetherEnum.NO.getCode()));
        if (animalImaging == null) {
            return null;
        }

        SampleAnimalImagingVo animalImagingVo = BeanUtil.copyProperties(animalImaging, SampleAnimalImagingVo.class);
        // 获取 活体成像信息
        LiveImagingDetail liveImagingDetail = liveImagingDetailService.getOne(new LambdaQueryWrapper<>(LiveImagingDetail.class)
                .eq(LiveImagingDetail::getImagingId, animalImaging.getId()));
        if (liveImagingDetail != null) {
            LiveImagingVo liveImagingVo = BeanUtil.copyProperties(liveImagingDetail, LiveImagingVo.class);
            animalImagingVo.setLiveImaging(liveImagingVo);
        }
        // 获取Micro-CT信息
        MicroCtDetail microCtDetail = microCtDetailService.getOne(new LambdaQueryWrapper<>(MicroCtDetail.class)
                .eq(MicroCtDetail::getImagingId, animalImaging.getId()));
        if (microCtDetail != null) {
            MicroCtVo microCtVo = BeanUtil.copyProperties(microCtDetail, MicroCtVo.class);
            animalImagingVo.setMicroCt(microCtVo);
        }
        // 获取超声信息
        UltrasoundDetail ultrasoundDetail = ultrasoundDetailService.getOne(new LambdaQueryWrapper<>(UltrasoundDetail.class)
                .eq(UltrasoundDetail::getImagingId, animalImaging.getId()));
        if (ultrasoundDetail != null) {
            UltrasoundVo ultrasoundVo = BeanUtil.copyProperties(ultrasoundDetail, UltrasoundVo.class);
            animalImagingVo.setUltrasound(ultrasoundVo);
        }

        return animalImagingVo;
    }
}