package com.github.jfcloud.gene.sample.vo;

import com.alibaba.fastjson.JSONArray;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;
import java.time.LocalDate;

@Data
public class SampleSubmitVo {

    @Schema(description = "项目名称")
    @Size(max = 255, message = "项目名称长度不能超过255个字符")
    private String projectName;

    @Schema(description = "项目编号")
    @Size(max = 255, message = "项目编号长度不能超过255个字符")
    private String projectNo;

    @Schema(description = "类型", example = "病理pathological 动物样本检测animal 动物影像检测animalImage")
    @NotEmpty(message = "类型不能为空")
    @Pattern(regexp = "pathological|animal|animalImage", message = "未知类型 type")
    private String type;

    @Schema(description = "申请人id")
    private Long applicantId;

    @Schema(description = "申请人名称")
    private String applicantName;

    @Schema(description = "申请人电话")
    @Size(max = 50, message = "申请人电话长度不能超过50个字符")
    private String applicantPhone;

    @Schema(description = "申请人邮箱")
    @Size(max = 100, message = "申请人邮箱长度不能超过100个字符")
    private String applicantEmail;

    @Schema(description = "申请时间", example = "2022-01-01")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDate applyTime;

    @Schema(description = "检测人员", example = "[{\"id\":1,\"name\":\"张三\"},{\"id\":2,\"name\":\"李四\"}]")
    private JSONArray inspector = new JSONArray();

    @Schema(description = "备注")
    private String remark;

    @Schema(description = "部门负责人Id")
    @NotNull(message = "部门负责人Id不能为空")
    private Long departLeaderId;

    @Schema(description = "部门负责人姓名")
    private String departLeaderName;

    @Schema(description = "项目管理部审核人")
    @NotNull(message = "项目管理部审核人不能为空")
    private Long projectManageId;

    @Schema(description = "项目管理部审核人姓名")
    private String projectManageName;

    private SampleAnimalVo animal;

    private SamplePathologicalVo pathological;

    private SampleAnimalImagingVo animalImage;
}