package com.github.jfcloud.gene.util;

import cn.hutool.core.util.StrUtil;
import freemarker.template.Configuration;
import lombok.extern.slf4j.Slf4j;
import sun.misc.BASE64Encoder;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.Map;

@Slf4j
public class WordUtil {

    /**
     * @param modelName 模板的名称 比如"basic.ftl"
     * @param dataMap   传入的数据（key=ftl 中的站位的名称同时 要是String ）
     */
    public static ByteArrayOutputStream exportWordOut(String modelName, Map<String, Object> dataMap) {
        if (dataMap != null) {
            for (Map.Entry<String, Object> entry : dataMap.entrySet()) {
                if (entry.getValue() == null) {
                    entry.setValue("");
                }
            }
        }

        // freemaker配置
        Configuration configuration = new Configuration(Configuration.VERSION_2_3_32);
        configuration.setDefaultEncoding("utf-8");
        configuration.setClassForTemplateLoading(WordUtil.class, "/ftlTemplate");

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try (Writer out = new BufferedWriter(new OutputStreamWriter(outputStream, StandardCharsets.UTF_8))){
            configuration.getTemplate(modelName).process(dataMap, out);
            return outputStream;
        } catch (Exception e) {
            log.error("导出word异常", e);
        }
        return null;
    }

    /**
     * @param modelName 模板的名称 比如"basic.ftl"
     * @param dataMap   传入的数据（key=ftl 中的站位的名称同时 要是String ）
     */
    public static InputStream exportWord(String modelName, Map<String, Object> dataMap) {
        try (ByteArrayOutputStream outputStream = exportWordOut(modelName, dataMap)){
            if (outputStream != null) {
                return new ByteArrayInputStream(outputStream.toByteArray());
            }
        } catch (Exception e) {
            log.error("导出word异常", e);
        }
        return null;
    }

    /**
     * 图片转换
     * @param inputStream
     * @return
     */
    public static String getImgFileToBase64(InputStream inputStream) {
        //将图片文件转化为字节数组字符串，并对其进行Base64编码处理
        byte[] buffer = null;
        //读取图片字节数组
        try {
            int count = 0;
            while (count == 0) {
                count = inputStream.available();
            }
            buffer = new byte[count];
            inputStream.read(buffer);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } finally {
            if (inputStream != null) {
                try {
                    // 关闭inputStream流
                    inputStream.close();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
        // 对字节数组Base64编码
        return new BASE64Encoder().encode(buffer);
    }

    /**
     * 在字符串左右填充字符串
     *
     * @param content 字符串
     * @param left    左侧空格
     * @param right   右侧空格
     * @return 填充后的字符串
     */
    public static String padString(String content, int left, int right) {
        if (content == null) {
            content = "";
        }
        return StrUtil.repeat(' ', left) + content + StrUtil.repeat(' ', right);
    }

    /**
     * 在字符串左右填充字符串
     *
     * @param content 字符串
     * @param length  需要达到的长度
     * @return 填充后的字符串
     */
    public static String padString(String content, int length) {
        if (content == null) {
            content = StrUtil.repeat(' ', length);
        }
        if (content.length() >= length) {
            return content;
        }
        int space = (length - content.length() / 2) / 2;
        return padString(content, space, space);
    }
}

